Index: src/Fl_lock.cxx
===================================================================
--- src/Fl_lock.cxx	(revision 5690)
+++ src/Fl_lock.cxx	(working copy)
@@ -68,6 +68,10 @@
 
 void	(*Fl::awake_cb)(void *);
 
+void Fl::awake(void* msg) {
+  do_awake(msg);
+}
+
 ////////////////////////////////////////////////////////////////
 // Windows threading...
 #ifdef WIN32
@@ -132,8 +136,9 @@
 // When called from a thread, it causes FLTK to awake from Fl::wait()...
 //
 
-void Fl::awake(void* msg) {
-  PostThreadMessage( main_thread, fl_wake_msg, (WPARAM)msg, 0);
+bool Fl::do_awake(void* msg) {
+  // TODO: ensure at least one fl_wake_msg is delivered
+  return PostThreadMessage( main_thread, fl_wake_msg, (WPARAM)msg, 0);
 }
 
 ////////////////////////////////////////////////////////////////
@@ -189,8 +194,8 @@
 }
 #  endif // PTHREAD_MUTEX_RECURSIVE
 
-void Fl::awake(void* msg) {
-  write(thread_filedes[1], &msg, sizeof(void*));
+bool Fl::do_awake(void* msg) {
+  return (write(thread_filedes[1], &msg, sizeof(void*)) > 0);
 }
 
 static void* thread_message_;
@@ -201,8 +206,9 @@
 }
 
 static void thread_awake_cb(int fd, void*) {
-  read(fd, &thread_message_, sizeof(void*));
-  if (Fl::awake_cb) (*Fl::awake_cb)(thread_message_);
+  while(read(fd, &thread_message_, sizeof(void*)) > 0) {
+    if (Fl::awake_cb) (*Fl::awake_cb)(thread_message_);
+  }
 }
 
 // These pointers are in Fl_x.cxx:
@@ -217,6 +223,8 @@
 
     // Make the write side of the pipe non-blocking to avoid deadlock
     // conditions (STR #1537)
+    fcntl(thread_filedes[0], F_SETFL,
+          fcntl(thread_filedes[0], F_GETFL) | O_NONBLOCK);
     fcntl(thread_filedes[1], F_SETFL,
           fcntl(thread_filedes[1], F_GETFL) | O_NONBLOCK);
 
Index: FL/Fl.H
===================================================================
--- FL/Fl.H	(revision 5690)
+++ FL/Fl.H	(working copy)
@@ -267,6 +267,7 @@
   static void lock();
   static void unlock();
   static void awake(void* message = 0);
+  static bool do_awake(void* message = 0);
   static void set_awake_cb(void (*cb)(void *)) { awake_cb = cb; }
   static void* thread_message();
 
